<?php

/**
 * Test: NLatteMacros::formatArray()
 *
 * @author     David Grudl
 * @package    Nette\Templates
 * @subpackage UnitTests
 */



require dirname(__FILE__) . '/../bootstrap.php';



// symbols

Assert::same( '',  NLatteMacros::formatArray('') );
Assert::same( '',  NLatteMacros::formatArray('', '&') );
Assert::same( 'array(1)',  NLatteMacros::formatArray('1') );
Assert::same( '&array(1)',  NLatteMacros::formatArray('1', '&') );
Assert::same( "array('symbol')",  NLatteMacros::formatArray('symbol') );
Assert::same( "array(1, 2, 'symbol1', 'symbol2')",  NLatteMacros::formatArray('1, 2, symbol1, symbol2') );

// strings

Assert::same( 'array("\"1, 2, symbol1, symbol2")',  NLatteMacros::formatArray('"\"1, 2, symbol1, symbol2"') ); // unable to parse "${'"'}" yet
Assert::same( "array('\\'1, 2, symbol1, symbol2')",  NLatteMacros::formatArray("'\\'1, 2, symbol1, symbol2'") );
try {
	NLatteMacros::formatArray("'\\\\'1, 2, symbol1, symbol2'");
	Assert::fail('Expected exception');
} catch (Exception $e) {
	Assert::exception('NTokenizerException', 'Unexpected %a% on line 1, column 27.', $e );
}

// key words

Assert::same( 'array(TRUE, false, null, 1 or 1 and 2 xor 3, clone $obj, new Class)',  NLatteMacros::formatArray('TRUE, false, null, 1 or 1 and 2 xor 3, clone $obj, new Class') );
Assert::same( 'array(func (10))',  NLatteMacros::formatArray('func (10)') );

// associative arrays

Assert::same( "array('symbol1' => 'value','symbol2'=>'value')",  NLatteMacros::formatArray('symbol1 => value,symbol2=>value') );
Assert::same( "array('symbol1' => array ('symbol2' => 'value'))",  NLatteMacros::formatArray('symbol1 => array (symbol2 => value)') );

// special

Assert::same( 'array($var)',  NLatteMacros::formatArray('$var') );
Assert::same( 'array($var => $var)',  NLatteMacros::formatArray('$var => $var') );
Assert::same( "array('symbol' => Class::CONST, )",  NLatteMacros::formatArray('symbol => Class::CONST, ') );
Assert::same( "array('symbol' => Class::method(), )",  NLatteMacros::formatArray('symbol => Class::method(), ') );
Assert::same( "array('symbol' => Namespace\\Class::method())",  NLatteMacros::formatArray('symbol => Namespace\Class::method()') );
Assert::same( "array('symbol' => Namespace \\ Class :: method ())",  NLatteMacros::formatArray('symbol => Namespace \ Class :: method ()') );
Assert::same( "array('symbol' => \$this->var, )",  NLatteMacros::formatArray('symbol => $this->var, ') );
Assert::same( "array('symbol' => \$this -> var, )",  NLatteMacros::formatArray('symbol => $this -> var, ') );
Assert::same( "array('symbol' => \$this -> var)",  NLatteMacros::formatArray('symbol => $this -> var') );
Assert::same( "array('symbol1' => 'value')",  NLatteMacros::formatArray('symbol1 => /*value,* /symbol2=>*/value/**/') );
